/*
 * Copyright (C) 2012 Brook Heisler
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in the
 * Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package com.castlebravostudios.listenersupport;

import java.lang.reflect.Method;
import javax.swing.SwingUtilities;

/**
 * This is an InvocationHandler that ensures that listener methods are always called
 * asynchronously on the event dispatch thread.
 */
class EdtLaterInvocationHandler<T> extends DefaultInvocationHandler<T> {

    public EdtLaterInvocationHandler( Iterable<T> listeners ) {
        super( listeners );
    }

    @Override
    protected void doIteration( final Method method, final Object[] args ) throws Throwable {
        Runnable edtRunnable = new InvocationHandlerRunnable( method, args );
        SwingUtilities.invokeLater( edtRunnable );
    }
}
